@@ -2394,8 +2394,11 @@ def test_newp_from_bytearray_doesnt_work():
 
 # XXX hack
 if sys.version_info >= (3,):
-    import posix, io
-    posix.fdopen = io.open
+    try:
+        import posix, io
+        posix.fdopen = io.open
+    except ImportError:
+        pass   # win32
 
 def test_FILE():
     if sys.platform == "win32":
@@ -2488,7 +2491,7 @@ def test_GetLastError():
     if sys.platform != "win32":
         py.test.skip("GetLastError(): only for Windows")
     #
-    lib = find_and_load_library('KERNEL32')
+    lib = find_and_load_library('KERNEL32.DLL')
     BInt = new_primitive_type("int")
     BVoid = new_void_type()
     BFunc1 = new_function_type((BInt,), BVoid, False)
@@ -2508,3 +2511,8 @@ def test_nonstandard_integer_types():
                      'uint32_t', 'int64_t', 'uint64_t', 'intptr_t',
                      'uintptr_t', 'ptrdiff_t', 'size_t', 'ssize_t']:
         new_primitive_type(typename)    # works
+
+def test_cannot_convert_unicode_to_charp():
+    BCharP = new_pointer_type(new_primitive_type("char"))
+    BCharArray = new_array_type(BCharP, None)
+    py.test.raises(TypeError, newp, BCharArray, u+'foobar')
