@@ -36,14 +36,13 @@ class FFI(object):
         """
         from . import cparser, model
         if backend is None:
-            try:
-                import _cffi_backend as backend
-            except ImportError as e:
-                import warnings
-                warnings.warn("import _cffi_backend: %s\n"
-                              "Falling back to the ctypes backend." % (e,))
-                from . import backend_ctypes
-                backend = backend_ctypes.CTypesBackend()
+            # You need PyPy (>= 2.0 beta), or a CPython (>= 2.6) with
+            # _cffi_backend.so compiled.
+            import _cffi_backend as backend
+            # (If you insist you can also try to pass the option
+            # 'backend=backend_ctypes.CTypesBackend()', but don't
+            # rely on it!  It's probably not going to work well.)
+
         self._backend = backend
         self._parser = cparser.Parser()
         self._cached_btypes = {}
@@ -52,7 +51,6 @@ class FFI(object):
         self._function_caches = []
         self._libraries = []
         self._cdefsources = []
-        self._pointer_type_cache = {}
         if hasattr(backend, 'set_ffi'):
             backend.set_ffi(self)
         for name in backend.__dict__:
@@ -294,17 +292,17 @@ class FFI(object):
     errno = property(_get_errno, _set_errno, None,
                      "the value of 'errno' from/to the C calls")
 
+    def _pointer_to(self, ctype):
+        from . import model
+        return model.pointer_cache(self, ctype)
+
     def addressof(self, cdata, field=None):
         """Return the address of a <cdata 'struct-or-union'>.
         If 'field' is specified, return the address of this field.
         """
         ctype = self._backend.typeof(cdata)
         ctype, offset = self._backend.typeoffsetof(ctype, field)
-        try:
-            ctypeptr = self._pointer_type_cache[ctype]
-        except KeyError:
-            ctypeptr = self._pointer_type_cache[ctype] = (
-                self._typeof(self.getctype(ctype, '*')))
+        ctypeptr = self._pointer_to(ctype)
         return self._backend.rawaddressof(ctypeptr, cdata, offset)
 
 
