@@ -1,11 +1,20 @@
 
 from . import api, model
-import pycparser, weakref, re
+import pycparser.c_parser, weakref, re, sys
+
+try:
+    if sys.version_info < (3,):
+        import thread as _thread
+    else:
+        import _thread
+    lock = _thread.allocate_lock()
+except ImportError:
+    lock = None
 
 _r_comment = re.compile(r"/\*.*?\*/|//.*?$", re.DOTALL | re.MULTILINE)
 _r_define  = re.compile(r"^\s*#\s*define\s+([A-Za-z_][A-Za-z_0-9]*)\s+(.*?)$",
                         re.MULTILINE)
-_r_partial_enum = re.compile(r"\.\.\.\s*\}")
+_r_partial_enum = re.compile(r"=\s*\.\.\.\s*[,}]|\.\.\.\s*\}")
 _r_enum_dotdotdot = re.compile(r"__dotdotdot\d+__$")
 _r_partial_array = re.compile(r"\[\s*\.\.\.\s*\]")
 _parser_cache = None
@@ -30,18 +39,30 @@ def _preprocess(csource):
     csource = _r_partial_array.sub('[__dotdotdotarray__]', csource)
     # Replace "...}" with "__dotdotdotNUM__}".  This construction should
     # occur only at the end of enums; at the end of structs we have "...;}"
-    # and at the end of vararg functions "...);"
+    # and at the end of vararg functions "...);".  Also replace "=...[,}]"
+    # with ",__dotdotdotNUM__[,}]": this occurs in the enums too, when
+    # giving an unknown value.
     matches = list(_r_partial_enum.finditer(csource))
     for number, match in enumerate(reversed(matches)):
         p = match.start()
-        assert csource[p:p+3] == '...'
-        csource = '%s __dotdotdot%d__ %s' % (csource[:p], number,
-                                             csource[p+3:])
+        if csource[p] == '=':
+            p2 = csource.find('...', p, match.end())
+            assert p2 > p
+            csource = '%s,__dotdotdot%d__ %s' % (csource[:p], number,
+                                                 csource[p2+3:])
+        else:
+            assert csource[p:p+3] == '...'
+            csource = '%s __dotdotdot%d__ %s' % (csource[:p], number,
+                                                 csource[p+3:])
     # Replace all remaining "..." with the same name, "__dotdotdot__",
     # which is declared with a typedef for the purpose of C parsing.
     return csource.replace('...', ' __dotdotdot__ '), macros
 
 class Parser(object):
+    TYPEDEFS = sorted(
+        [_name for _name in model.PrimitiveType.ALL_PRIMITIVE_TYPES
+               if _name.endswith('_t')])
+
     def __init__(self):
         self._declarations = {}
         self._anonymous_counter = 0
@@ -53,17 +74,44 @@ class Parser(object):
         # internals of CParser...  the following registers the
         # typedefs, because their presence or absence influences the
         # parsing itself (but what they are typedef'ed to plays no role)
-        csourcelines = ['typedef int wchar_t;']
+        typenames = self.TYPEDEFS[:]
         for name in sorted(self._declarations):
             if name.startswith('typedef '):
-                csourcelines.append('typedef int %s;' % (name[8:],))
+                typenames.append(name[8:])
+        csourcelines = ['typedef int %s;' % typename for typename in typenames]
         csourcelines.append('typedef int __dotdotdot__;')
         csource, macros = _preprocess(csource)
         csourcelines.append(csource)
         csource = '\n'.join(csourcelines)
-        ast = _get_parser().parse(csource)
+        if lock is not None:
+            lock.acquire()     # pycparser is not thread-safe...
+        try:
+            ast = _get_parser().parse(csource)
+        except pycparser.c_parser.ParseError as e:
+            self.convert_pycparser_error(e, csource)
+        finally:
+            if lock is not None:
+                lock.release()
         return ast, macros
 
+    def convert_pycparser_error(self, e, csource):
+        # xxx look for ":NUM:" at the start of str(e) and try to interpret
+        # it as a line number
+        line = None
+        msg = str(e)
+        if msg.startswith(':') and ':' in msg[1:]:
+            linenum = msg[1:msg.find(':',1)]
+            if linenum.isdigit():
+                linenum = int(linenum, 10)
+                csourcelines = csource.splitlines()
+                if 1 <= linenum <= len(csourcelines):
+                    line = csourcelines[linenum-1]
+        if line:
+            msg = 'cannot parse "%s"\n%s' % (line, msg)
+        else:
+            msg = 'parse error\n%s' % (msg,)
+        raise api.CDefError(msg)
+
     def parse(self, csource, override=False):
         prev_override = self._override
         try:
@@ -98,6 +146,12 @@ class Parser(object):
                 if (isinstance(decl.type.type, pycparser.c_ast.IdentifierType)
                         and decl.type.type.names == ['__dotdotdot__']):
                     realtype = model.unknown_type(decl.name)
+                elif (isinstance(decl.type, pycparser.c_ast.PtrDecl) and
+                      isinstance(decl.type.type, pycparser.c_ast.TypeDecl) and
+                      isinstance(decl.type.type.type,
+                                 pycparser.c_ast.IdentifierType) and
+                      decl.type.type.type.names == ['__dotdotdot__']):
+                    realtype = model.unknown_ptr_type(decl.name)
                 else:
                     realtype = self._get_type(decl.type, name=decl.name)
                 self._declare('typedef ' + decl.name, realtype)
@@ -115,19 +169,19 @@ class Parser(object):
             if isinstance(node, pycparser.c_ast.Struct):
                 # XXX do we need self._declare in any of those?
                 if node.decls is not None:
-                    self._get_struct_or_union_type('struct', node)
+                    self._get_struct_union_enum_type('struct', node)
             elif isinstance(node, pycparser.c_ast.Union):
                 if node.decls is not None:
-                    self._get_struct_or_union_type('union', node)
+                    self._get_struct_union_enum_type('union', node)
             elif isinstance(node, pycparser.c_ast.Enum):
                 if node.values is not None:
-                    self._get_enum_type(node)
+                    self._get_struct_union_enum_type('enum', node)
             elif not decl.name:
                 raise api.CDefError("construct does not declare any variable",
                                     decl)
             #
             if decl.name:
-                tp = self._get_type(node)
+                tp = self._get_type(node, partial_length_ok=True)
                 if self._is_constant_declaration(node):
                     self._declare('constant ' + decl.name, tp)
                 else:
@@ -148,7 +202,7 @@ class Parser(object):
                 raise api.FFIError(
                     "multiple declarations of %s (for interactive usage, "
                     "try cdef(xx, override=True))" % (name,))
-        assert name != '__dotdotdot__'
+        assert '__dotdotdot__' not in name.split()
         self._declarations[name] = obj
 
     def _get_type_pointer(self, type, const=False):
@@ -215,15 +269,15 @@ class Parser(object):
             #
             if isinstance(type, pycparser.c_ast.Struct):
                 # 'struct foobar'
-                return self._get_struct_or_union_type('struct', type, name)
+                return self._get_struct_union_enum_type('struct', type, name)
             #
             if isinstance(type, pycparser.c_ast.Union):
                 # 'union foobar'
-                return self._get_struct_or_union_type('union', type, name)
+                return self._get_struct_union_enum_type('union', type, name)
             #
             if isinstance(type, pycparser.c_ast.Enum):
                 # 'enum foobar'
-                return self._get_enum_type(type)
+                return self._get_struct_union_enum_type('enum', type, name)
         #
         if isinstance(typenode, pycparser.c_ast.FuncDecl):
             # a function type
@@ -232,6 +286,14 @@ class Parser(object):
                 result = result.as_function_pointer()
             return result
         #
+        # nested anonymous structs or unions end up here
+        if isinstance(typenode, pycparser.c_ast.Struct):
+            return self._get_struct_union_enum_type('struct', typenode, name,
+                                                    nested=True)
+        if isinstance(typenode, pycparser.c_ast.Union):
+            return self._get_struct_union_enum_type('union', typenode, name,
+                                                    nested=True)
+        #
         raise api.FFIError("bad or unsupported type declaration")
 
     def _parse_function_type(self, typenode, funcname=None):
@@ -244,7 +306,11 @@ class Parser(object):
             params[-1].type.type.names == ['__dotdotdot__'])
         if ellipsis:
             params.pop()
-        if (len(params) == 1 and
+            if not params:
+                raise api.CDefError(
+                    "%s: a function with only '(...)' as argument"
+                    " is not correct C" % (funcname or 'in expression'))
+        elif (len(params) == 1 and
             isinstance(params[0].type, pycparser.c_ast.TypeDecl) and
             isinstance(params[0].type.type, pycparser.c_ast.IdentifierType)
                 and list(params[0].type.type.names) == ['void']):
@@ -266,7 +332,7 @@ class Parser(object):
             return const or 'const' in typenode.quals
         return False
 
-    def _get_struct_or_union_type(self, kind, type, name=None):
+    def _get_struct_union_enum_type(self, kind, type, name=None, nested=False):
         # First, a level of caching on the exact 'type' node of the AST.
         # This is obscure, but needed because pycparser "unrolls" declarations
         # such as "typedef struct { } foo_t, *foo_p" and we end up with
@@ -311,16 +377,27 @@ class Parser(object):
                 tp = model.StructType(explicit_name, None, None, None)
             elif kind == 'union':
                 tp = model.UnionType(explicit_name, None, None, None)
+            elif kind == 'enum':
+                tp = self._build_enum_type(explicit_name, type.values)
             else:
                 raise AssertionError("kind = %r" % (kind,))
             if name is not None:
                 self._declare(key, tp)
+        else:
+            if kind == 'enum' and type.values is not None:
+                raise NotImplementedError(
+                    "enum %s: the '{}' declaration should appear on the first "
+                    "time the enum is mentioned, not later" % explicit_name)
         tp.forcename = tp.forcename or force_name
         if tp.forcename and '$' in tp.name:
             self._declare('anonymous %s' % tp.forcename, tp)
         #
         self._structnode2type[type] = tp
         #
+        # enums: done here
+        if kind == 'enum':
+            return tp
+        #
         # is there a 'type.decls'?  If yes, then this is the place in the
         # C sources that declare the fields.  If no, then just return the
         # existing type, possibly still incomplete.
@@ -338,7 +415,7 @@ class Parser(object):
                 # XXX pycparser is inconsistent: 'names' should be a list
                 # of strings, but is sometimes just one string.  Use
                 # str.join() as a way to cope with both.
-                self._make_partial(tp)
+                self._make_partial(tp, nested)
                 continue
             if decl.bitsize is None:
                 bitsize = -1
@@ -347,27 +424,33 @@ class Parser(object):
             self._partial_length = False
             type = self._get_type(decl.type, partial_length_ok=True)
             if self._partial_length:
-                self._make_partial(tp)
-            fldnames.append(decl.name)
+                self._make_partial(tp, nested)
+            if isinstance(type, model.StructType) and type.partial:
+                self._make_partial(tp, nested)
+            fldnames.append(decl.name or '')
             fldtypes.append(type)
             fldbitsize.append(bitsize)
         tp.fldnames = tuple(fldnames)
         tp.fldtypes = tuple(fldtypes)
         tp.fldbitsize = tuple(fldbitsize)
+        if fldbitsize != [-1] * len(fldbitsize):
+            if isinstance(tp, model.StructType) and tp.partial:
+                raise NotImplementedError("%s: using both bitfields and '...;'"
+                                          % (tp,))
         return tp
 
-    def _make_partial(self, tp):
+    def _make_partial(self, tp, nested):
         if not isinstance(tp, model.StructType):
             raise api.CDefError("%s cannot be partial" % (tp,))
-        if not tp.has_c_name():
-            raise api.CDefError("%s is partial but has no C name" % (tp,))
+        if not tp.has_c_name() and not nested:
+            raise NotImplementedError("%s is partial but has no C name" %(tp,))
         tp.partial = True
 
     def _parse_constant(self, exprnode, partial_length_ok=False):
         # for now, limited to expressions that are an immediate number
         # or negative number
         if isinstance(exprnode, pycparser.c_ast.Constant):
-            return int(exprnode.value)
+            return int(exprnode.value, 0)
         #
         if (isinstance(exprnode, pycparser.c_ast.UnaryOp) and
                 exprnode.op == '-'):
@@ -382,34 +465,12 @@ class Parser(object):
         raise api.FFIError("unsupported non-constant or "
                            "not immediately constant expression")
 
-    def _get_enum_type(self, type):
-        # See _get_struct_or_union_type() for the reason of the
-        # complicated logic here.  This is still a simplified version,
-        # assuming that it's ok to assume the more complicated cases
-        # don't occur...
-        try:
-            return self._structnode2type[type]
-        except KeyError:
-            pass
-        name = type.name
-        if name is None:
-            self._anonymous_counter += 1
-            explicit_name = '$%d' % self._anonymous_counter
-            key = None
-        else:
-            explicit_name = name
-            key = 'enum %s' % (name,)
-            tp = self._declarations.get(key, None)
-            if tp is not None:
-                return tp
-        #
-        decls = type.values
+    def _build_enum_type(self, explicit_name, decls):
         if decls is not None:
-            enumerators = [enum.name for enum in decls.enumerators]
-            partial = False
-            if enumerators and _r_enum_dotdotdot.match(enumerators[-1]):
-                enumerators.pop()
-                partial = True
+            enumerators1 = [enum.name for enum in decls.enumerators]
+            enumerators = [s for s in enumerators1
+                             if not _r_enum_dotdotdot.match(s)]
+            partial = len(enumerators) < len(enumerators1)
             enumerators = tuple(enumerators)
             enumvalues = []
             nextenumvalue = 0
@@ -421,11 +482,6 @@ class Parser(object):
             enumvalues = tuple(enumvalues) 
             tp = model.EnumType(explicit_name, enumerators, enumvalues)
             tp.partial = partial
-            if key is not None:
-                self._declare(key, tp)
         else:   # opaque enum
-            enumerators = ()
-            enumvalues = ()
             tp = model.EnumType(explicit_name, (), ())
-        self._structnode2type[type] = tp
         return tp
