@@ -39,7 +39,12 @@ class VGenericEngine(object):
         # 'export_symbols', so instead of fighting it, just give up and
         # give it one
         if sys.platform == 'win32':
-            prnt("void init%s(void) { }\n" % self.verifier.get_module_name())
+            if sys.version_info >= (3,):
+                prefix = 'PyInit_'
+            else:
+                prefix = 'init'
+            modname = self.verifier.get_module_name()
+            prnt("void %s%s(void) { }\n" % (prefix, modname))
 
     def load_library(self):
         # import it with the CFFI backend
@@ -61,8 +66,11 @@ class VGenericEngine(object):
         self._load(module, 'loaded', library=library)
         return library
 
+    def _get_declarations(self):
+        return sorted(self.ffi._parser._declarations.items())
+
     def _generate(self, step_name):
-        for name, tp in self.ffi._parser._declarations.items():
+        for name, tp in self._get_declarations():
             kind, realname = name.split(' ', 1)
             try:
                 method = getattr(self, '_generate_gen_%s_%s' % (kind,
@@ -73,7 +81,7 @@ class VGenericEngine(object):
             method(tp, realname)
 
     def _load(self, module, step_name, **kwds):
-        for name, tp in self.ffi._parser._declarations.items():
+        for name, tp in self._get_declarations():
             kind, realname = name.split(' ', 1)
             method = getattr(self, '_%s_gen_%s' % (step_name, kind))
             method(tp, realname, module, **kwds)
